"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkspaceData = void 0;
const ide_core_common_1 = require("@opensumi/ide-core-common");
const workspaceSchema = {
    type: 'object',
    properties: {
        folders: {
            description: 'Root folders in the workspace',
            type: 'array',
            items: {
                type: 'object',
                properties: {
                    path: {
                        type: 'string',
                    },
                },
                required: ['path'],
            },
        },
        settings: {
            description: 'Workspace preferences',
            type: 'object',
        },
    },
    required: ['folders'],
};
var WorkspaceData;
(function (WorkspaceData) {
    let validateSchema;
    function is(data) {
        if (!validateSchema) {
            // 避免一开始就加载 ajv，初始化和 compile 都会花费大量的时间进行编译
            const Ajv = require('ajv');
            validateSchema = new Ajv().compile(workspaceSchema);
        }
        return !!validateSchema(data);
    }
    WorkspaceData.is = is;
    function buildWorkspaceData(folders, settings) {
        let roots = [];
        if (folders.length > 0) {
            if (typeof folders[0] !== 'string') {
                roots = folders.map((folder) => folder.uri);
            }
            else {
                roots = folders;
            }
        }
        const data = {
            folders: roots.map((folder) => ({ path: folder })),
        };
        if (settings) {
            data.settings = settings;
        }
        return data;
    }
    WorkspaceData.buildWorkspaceData = buildWorkspaceData;
    /**
     * 存储时需要将 {workspace}.sumi-workspace 内容存储为相对路径存储
     * 1. 如果可用相对路径表示，则存储为:
     * {
     *  "folders": [
     *    {
     *       "path": "folder1"
     *     },
     *     {
     *       "path": "folder2"
     *     }
     *   ],
     * }
     *
     * 2. 如果不可用相对路径表示，则存储绝对路径：
     * {
     *  "folders": [
     *    {
     *       "path": "file://abc/folder1"
     *     },
     *     {
     *       "path": "file://cdf/folder2"
     *     }
     *   ],
     * }
     *
     * @export
     * @param {WorkspaceData} data
     * @param {FileStat} [workspaceFile]
     * @returns {WorkspaceData}
     */
    function transformToRelative(data, workspaceFile) {
        const folderUris = [];
        const workspaceFileUri = new ide_core_common_1.URI(workspaceFile ? workspaceFile.uri : '').withScheme(ide_core_common_1.Schemes.file);
        for (const { path } of data.folders) {
            const folderUri = new ide_core_common_1.URI(path).withScheme(ide_core_common_1.Schemes.file);
            if (workspaceFileUri.parent.isEqualOrParent(folderUri)) {
                if (workspaceFileUri.parent.isEqual(folderUri)) {
                    folderUris.push('.');
                }
                else {
                    const rel = workspaceFileUri.parent.relative(folderUri);
                    folderUris.push(rel.toString() || '.');
                }
            }
            else {
                folderUris.push(folderUri.toString());
            }
        }
        return buildWorkspaceData(folderUris, data.settings);
    }
    WorkspaceData.transformToRelative = transformToRelative;
    /**
     * 将 {workspace}.sumi-workspace 中存储的相对路径转换为绝对路径
     *
     * @export
     * @param {WorkspaceData} data
     * @param {FileStat} [workspaceFile]
     * @returns {WorkspaceData}
     */
    function transformToAbsolute(data, workspaceFile) {
        if (workspaceFile) {
            const folders = [];
            for (const folder of data.folders) {
                const path = folder.path;
                // 判断是否为绝对路径
                if (/^.+:\//.test(path)) {
                    folders.push(path);
                }
                else {
                    if (path === '.') {
                        folders.push(new ide_core_common_1.URI(workspaceFile.uri).parent.toString());
                    }
                    else {
                        folders.push(new ide_core_common_1.URI(workspaceFile.uri).parent.resolve(path).toString());
                    }
                }
            }
            return Object.assign(data, buildWorkspaceData(folders, data.settings));
        }
        return data;
    }
    WorkspaceData.transformToAbsolute = transformToAbsolute;
})(WorkspaceData = exports.WorkspaceData || (exports.WorkspaceData = {}));
//# sourceMappingURL=workspace-data.js.map