import { AppConfig, DisposableCollection, Emitter, Event, IDisposable, ILogger, IWindowService, PreferenceSchemaProvider, PreferenceService } from '@opensumi/ide-core-browser';
import { URI } from '@opensumi/ide-core-common';
import { FileStat } from '@opensumi/ide-file-service';
import { IFileServiceClient } from '@opensumi/ide-file-service/lib/common';
import { IWorkspaceService, WorkspaceInput } from '../common';
import { WorkspaceData } from './workspace-data';
import { WorkspacePreferences } from './workspace-preferences';
export declare class WorkspaceService implements IWorkspaceService {
    private _workspace;
    private _roots;
    private deferredRoots;
    protected readonly fileServiceClient: IFileServiceClient;
    protected readonly windowService: IWindowService;
    protected logger: ILogger;
    protected preferences: WorkspacePreferences;
    protected preferenceService: PreferenceService;
    protected readonly schemaProvider: PreferenceSchemaProvider;
    protected readonly appConfig: AppConfig;
    private readonly storageProvider;
    private recentGlobalStorage;
    private readonly clientApp;
    get workspaceSuffixName(): string;
    protected applicationName: string;
    private _whenReady;
    protected readonly toDisposableCollection: DisposableCollection;
    private workspaceToName;
    init(): void;
    initFileServiceExclude(): Promise<void>;
    get whenReady(): Promise<void>;
    protected doInit(): Promise<void>;
    protected getTemporaryWorkspaceFileUri(home: URI): URI;
    protected listenPreference(): void;
    protected setFileServiceExcludes(): Promise<void>;
    protected getFlattenExcludes(name: string): string[];
    /**
     * 获取默认的workspace路径
     */
    protected getDefaultWorkspacePath(): string | undefined;
    get roots(): Promise<FileStat[]>;
    tryGetRoots(): FileStat[];
    get workspace(): FileStat | undefined;
    protected readonly onWorkspaceChangeEmitter: Emitter<FileStat[]>;
    get onWorkspaceChanged(): Event<FileStat[]>;
    protected readonly onWorkspaceFileExcludeChangeEmitter: Emitter<void>;
    get onWorkspaceFileExcludeChanged(): Event<void>;
    protected readonly onWorkspaceLocationChangedEmitter: Emitter<FileStat | undefined>;
    get onWorkspaceLocationChanged(): Event<FileStat | undefined>;
    protected readonly toDisposeOnWorkspace: DisposableCollection;
    setWorkspace(workspaceStat: FileStat | undefined): Promise<void>;
    protected updateWorkspace(): Promise<void>;
    protected updateRoots(): Promise<void>;
    protected computeRoots(): Promise<FileStat[]>;
    protected getWorkspaceDataFromFile(): Promise<WorkspaceData | undefined>;
    protected formatTitle(title?: string): string;
    protected updateTitle(): void;
    getMostRecentlyUsedWorkspace(): Promise<string | undefined>;
    setMostRecentlyUsedWorkspace(path: string): Promise<void>;
    getMostRecentlyUsedWorkspaces(): Promise<string[]>;
    getMostRecentlyUsedCommands(): Promise<string[]>;
    setMostRecentlyUsedCommand(commandId: string): Promise<void>;
    private getGlobalRecentStorage;
    /**
     * 当已经存在打开的工作区时，返回true
     * @returns {boolean}
     */
    get opened(): boolean;
    /**
     * 当一个混合工作区打开时，返回 true
     * @returns {boolean}
     */
    get isMultiRootWorkspaceOpened(): boolean;
    /**
     * 当前存在打开的工作区同时支持混合工作区时，返回true
     * @returns {boolean}
     */
    get isMultiRootWorkspaceEnabled(): boolean;
    /**
     * 打开一个文件夹或创建一个工作区
     * @param {URI} uri
     * @param {WorkspaceInput} [options]
     * @memberof WorkspaceService
     */
    open(uri: URI, options?: WorkspaceInput): Promise<void>;
    /**
     * 需要判断是否在当前工作区打开窗口
     * 在这里传入的options优先级高于preference设置
     * @protected
     * @param {URI} uri
     * @param {WorkspaceInput} [options]
     * @returns {Promise<void>}
     * @memberof WorkspaceService
     */
    protected doOpen(uri: URI, options?: WorkspaceInput): Promise<void>;
    /**
     * 为工作区设置根目录
     * @param uri
     */
    addRoot(uri: URI): Promise<void>;
    /**
     * 工作区中移除对应根目录
     */
    removeRoots(uris: URI[]): Promise<void>;
    spliceRoots(start: number, deleteCount?: number, workspaceToName?: {
        [key: string]: string;
    }, ...rootsToAdd: URI[]): Promise<URI[]>;
    getWorkspaceName(uri: URI): any;
    protected getUntitledWorkspace(): Promise<URI | undefined>;
    private writeWorkspaceFile;
    /**
     * 清理当前workspace
     */
    close(): Promise<void>;
    /**
     * 验证给定的URI是否为有效根目录
     */
    protected toValidRoot(uri: URI | string | undefined): Promise<FileStat | undefined>;
    /**
     * 返回文件的FileStat
     */
    protected toFileStat(uri: URI | string | undefined): Promise<FileStat | undefined>;
    protected openWindow(fileStat: FileStat, options?: WorkspaceInput): void;
    protected reloadWindow(): void;
    protected openNewWindow(workspacePath: string): void;
    protected shouldPreserveWindow(options?: WorkspaceInput): boolean;
    /**
     * 返回根目录下是否存在对应相对路径文件
     */
    containsSome(paths: string[]): Promise<boolean>;
    get saved(): boolean;
    /**
     * 存储工作区数据到文件中
     * @param uri URI or FileStat of the workspace file
     */
    save(uri: URI | FileStat): Promise<void>;
    protected readonly rootWatchers: Map<string, IDisposable>;
    protected watchRoots(): Promise<void>;
    protected watchRoot(root: FileStat): Promise<void>;
    /**
     * 根据给定的uri获取其根节点
     * 如果不指定uri，则获取默认的根节点
     * @param uri
     */
    getWorkspaceRootUri(uri: URI | undefined): URI | undefined;
    /**
     * 获取相对路径
     * @param pathOrUri
     * @param includeWorkspaceFolder
     */
    asRelativePath(pathOrUri: string | URI, includeWorkspaceFolder?: boolean): Promise<{
        path: string | undefined;
        root: string | undefined;
    }>;
    dispose(): void;
}
//# sourceMappingURL=workspace-service.d.ts.map